require "rake/testtask"
require "rake/packagetask"

$LOAD_PATH << File.join("./", "lib")
require "noms"
require "fileutils"

NAME = "noms"
PACKAGE_FILES = File.readlines("MANIFEST.txt").map { |f| f.chomp }.delete_if { |f| f.empty? }

# Make directory return a useful value
alias :original_directory :directory
def directory(dir)
  original_directory(dir)
  Rake::Task[dir]
end

task :clean_cache do
  rm_rf("tmp/cache")
end

task :loc do
  sh("(find bin lib && find public -depth 1) | egrep '.rb|.js$' | xargs wc -l")
end

namespace :web do
  FileList["public-raw/**/*"].each do |f|
    next if File.directory?(f)
    
    public_path = f.sub(/^public-raw/, "public")
    file(public_path => [directory(public_path.pathmap("%d")), f]) do
      extension = f.pathmap("%x")
      if extension.any? && %[.js .css].include?(extension)
        sh("java", "-jar", "vendor/yuicompressor-2.4.2.jar", "-o", public_path, f)
      else
        FileUtils.cp(f, public_path)
      end
    end
    desc("Deploy the public web directory, and minimize JS and CSS")
    task :public_directory => public_path
  end
  
  task :run_dev => :public_directory do
    ENV["NOMS_DEBUG"] = "true"
    sh("rackup config/config.ru")
  end
  
  desc("Starts the web-app for development")
  task :run  => [:clean_cache, :run_dev]
  
  desc("Updates caches for a NOMS site")
  task :caches do
    remote_site = ENV["SITE"] || "http://gridstats.com"
    sh("ruby", "bin/caches.rb", remote_site, *FileList["config/noms/*.{config,group}"])
  end
end

namespace :thin do
  task :restart => [:stop, :start]
  
  task :start  => [:clean_cache, "web:public_directory"] do
    sh("thin start --chdir `pwd` -C config/thin.yml")
  end
  
  task :stop do
    sh("thin stop -C config/thin.yml")
  end
end

namespace :db do
  desc("Starts a console for interactive development")
  task :console do
    sh("irb -Ilib -rnoms_irb")
  end
  
  task :gen_servers do
    FileList["config/noms/*.config"].each do |config|
      sh("ruby", "bin/stats.rb", config)
    end
  end
  
  task :gen_groups do
    FileList["config/noms/*.group"].each do |config|
      sh("ruby", "bin/group.rb", config)
    end
  end
  
  task :gen => [:gen_servers, :gen_groups]
end

Rake::TestTask.new do |t|
  t.test_files = FileList['test/test_*.rb']
end

Rake::PackageTask.new(NAME, File.read("VERSION.txt").chomp) do |p|
  p.need_zip = true
  p.package_files.include(*PACKAGE_FILES)
end

task("doc/config.txt" => [directory("doc")]) do
  c = NOMS::Config.new
  c.define_config
  File.open("doc/config.txt", "w") do |f|
    c.with_each do |name, doc, level|
      indent = "    " * level
      f.puts "#{indent}#{name}: #{doc}"
    end
  end
end

task :package => "doc/config.txt"
