# NOMS -- A web-app for Armagetron Advanced statistics
# Copyright (C) 2010  Daniel Harple
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation in version 3 of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# A program to update the caches for a NOMS site.

require "net/http"
require "uri"
require "yaml"

def fetch(location)
  uri = URI.parse(location)
  print "Fetching #{uri}... "
  STDOUT.flush
  res = Net::HTTP.get_response(uri)
  puts res.code
end

if __FILE__ == $PROGRAM_NAME
  if ARGV.length < 2
    puts "Usage: ruby #{$PROGRAM_NAME} <remote-site-address> <config-files ...>"
    exit(1)
  end
  
  remote_site = ARGV[0]
  configs = ARGV[1..-1]
  
  fetch("#{remote_site}/")
  configs.each do |config|
    short_name = File.open(config) { |f| YAML.load(f) }["short_name"]
    
    fetch("#{remote_site}/#{short_name}/world-map-players.json")
    (1..3).each do |tier|
      var = tier == 1 ? "" : "?tier=#{tier}"
      fetch("#{remote_site}/#{short_name}/stats#{var}")
    end
    fetch("#{remote_site}/#{short_name}/stats/best")
    fetch("#{remote_site}/#{short_name}/achievements")
    fetch("#{remote_site}/#{short_name}")
  end
end
