# NOMS -- A web-app for Armagetron Advanced statistics
# Copyright (C) 2010  Daniel Harple
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation in version 3 of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

require "rexml/document"
require "rexml/streamlistener"
require "rubygems"
require "json"

class WorldMapConverter
  include REXML::StreamListener
  
  def initialize(stream)
    @countries = []
    @in_group = false
    REXML::Document.parse_stream(stream, self)
  end
  
  def tag_start(name, attrs)
    return unless %w[polygon path g].member?(name)
    
    country = self.current_country
    
    case name
    when "polygon"
      country["paths"] << convert_polygon(attrs["points"])
      country["id"] = extract_id(attrs) if !self.in_group?
    when "path"
      country["paths"] << attrs["d"]
      country["id"] = extract_id(attrs) if !self.in_group?
    when "g"
      @in_group = true
      country["id"] = extract_id(attrs)
    end
  end
  
  def tag_end(name)
    @in_group = false if name == "g"
  end
  
  def to_json(stream=STDOUT)
    JSON.dump(@countries, stream)
  end
  
  protected
  
  def current_country
    return @countries[-1] if self.in_group?
    c = {"paths" => []}
    @countries << c
    return c
  end
  
  def in_group?
    @in_group
  end
  
  def convert_polygon(points)
    first_pair_index = points.index(/\s/)
    first_pair, rest = points[0..(first_pair_index - 1)], points[(first_pair_index + 1)..-1]
    return "M #{first_pair} L #{rest.rstrip}z"
  end
  
  def extract_id(attrs)
    id = attrs["id"] || ""
    id = "" if id.length > 2 # Only use country codes for IDs.
    return id
  end
end

def main(world_map_path)
  converter = File.open(world_map_path) { |f| WorldMapConverter.new(f) }
  converter.to_json
end

if __FILE__ == $PROGRAM_NAME
  main(ARGV[0])
end
