# NOMS -- A web-app for Armagetron Advanced statistics
# Copyright (C) 2010  Daniel Harple
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation in version 3 of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

$LOAD_PATH << File.join(File.dirname($PROGRAM_NAME), "..", "lib")

require "noms"
require "fileutils"

def main(group_config)
  group_server = NOMS::Setup.add_config(group_config)
  
  # Assume the configs are placed in the same directory, and that they are named
  # consistently.
  configs = group_server.config.servers_grouped.map { |short_name|
    File.join(File.dirname(group_config), short_name + ".config")
  }
  
  # No need to merge the first DB into an new empty one, just copy it
  first_config = configs.shift
  puts "Merging #{first_config}"
  first_server = NOMS::Setup.add_config(first_config)
  
  FileUtils.cp(first_server.db_path, group_server.db_path)
  
  # Use the newly copied DB
  group_server.connect_db
  NOMS::Setup.generate_models(false)
    
  # Clear out entries we don't need
  NOMS.set_current_server(group_server)
  unused_datasets = [
    NOMS::DB::Meta,
    NOMS::DB::PerfectMatchAchievement,
    NOMS::DB::MinimumMatchAchievement,
    NOMS::DB::MatchWinnerRoundLoserAchievement,
    NOMS::DB::MatchWinnerRoundWinnerAchievement,
    NOMS::DB::DominationAchievement
  ]
  unused_datasets.each do |klass|
    klass.select_all.delete
  end
  
  last_global_match_number = NOMS::DB::MatchWin.distinct_matches_count
  configs.each do |config|
    puts "Merging #{config}"
    server = NOMS::Setup.add_config(config)
    players = NOMS::DB::Player.all
    match_wins = NOMS::DB::MatchWin.order_by(:global_match_number).reverse.all
    global_match_numbers_mapping = {}
    
    NOMS.set_current_server(group_server)
    
    # We are merging Left into Right
    players.each do |left_player|
      next if group_server.config.sanity_checks.filter_no_rounds_played && left_player.rounds_played <= 0
      
      left_match_wins = match_wins.find_all { |win| win.player_id == left_player.id }
      right_player = NOMS::DB::Player[:name => left_player.name] || NOMS::Light::Player.new(left_player.name).to_heavy

      right_player.merge(left_player)
      right_player.won_matches += left_match_wins.length
      right_player.save_changes
      
      left_match_wins.each { |left_win|
        global_match_number = if global_match_numbers_mapping[left_win.global_match_number]
          global_match_numbers_mapping[left_win.global_match_number]
        else
          last_global_match_number += 1
          global_match_numbers_mapping[left_win.global_match_number] = last_global_match_number
        end
        
        right_win = NOMS::DB::MatchWin.new
        right_win.merge(left_win)
        right_win.global_match_number = global_match_number
        right_player.add_match_win(right_win)
      }
    end
  end
  
end

if __FILE__ == $PROGRAM_NAME
  if ARGV.length != 1
    puts "Usage: ruby #{$PROGRAM_NAME} <group-config>"
    exit(1)
  end
  
  main(ARGV[0])
end
