# NOMS -- A web-app for Armagetron Advanced statistics
# Copyright (C) 2009  Daniel Harple
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation in version 3 of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

require "builder"
require "open3"
require "rexml/document"
require "rexml/streamlistener"

module NOMS
  module ArmaSkill

    class Parser
      include REXML::StreamListener
      
      def initialize(stream, data)
        @data = data
        REXML::Document.parse_stream(stream, self)
      end
      
       def tag_start(name, attrs)
         case name
         when "player"
           @mu, @sigma, @rating = Float(attrs["mu"]), Float(attrs["sigma"]), Float(attrs["rating"])
         end
       end
       
      def text(player)
        @data[player].mu = @mu
        @data[player].sigma = @sigma
        @data[player].rating = @rating
        @data[player].is_armaskill_rated = true
       end
    end
    
    def self.process_skills(matches, data)
      matches_path = NOMS::Setup.database_file(false, "-matches", "xml")
      seeding_path = NOMS::Setup.database_file(false, "-seeding", "xml")
      export_matches(matches, matches_path)
      export_seeding(data, seeding_path)
      
      args = ["stddev-multiplier", "mean", "beta", "dynamics-factor", "draw-probability"].map { |name|
        "--#{name} #{NOMS.config.armaskill[name]}"
      }.join(" ")
      
      Open3.popen3("armaskill #{args} #{NOMS::Util.e_sh matches_path} #{NOMS::Util.e_sh seeding_path}") { |stdin, stdout, stderr|
        # FIXME Investigate why some versions of REXML can't process this stream fully.
        Parser.new(stdout.read, data)
      }
    end
    
    private
    
    def self.export_seeding(data, path)
      File.open(path, "w") do |f|
        builder = Builder::XmlMarkup.new(:target => f)
        builder.root { |root_node|
          data.each_key do |name|
            heavy_player = NOMS::DB::Player[:name => name]
            if heavy_player.nil? || !heavy_player.is_armaskill_rated || heavy_player.matches_played.zero?
              next
            end
            root_node.player(name, :mu => heavy_player.mu, :sigma => heavy_player.sigma)
          end
        }
      end
    end
    
    def self.export_matches(matches, path)
      File.open(path, "w") do |f|
        builder = Builder::XmlMarkup.new(:target => f)
        builder.root { |root|
          matches.each do |match|
            match.to_xml(root)
          end
        }
      end
    end
    
  end
end
