# NOMS -- A web-app for Armagetron Advanced statistics
# Copyright (C) 2009  Daniel Harple
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation in version 3 of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

module NOMS
  module StatFile
    
    class Parser
      def initialize(stream, &converter)
        @data = {}
        process(stream, &converter)
      end
      
      attr_reader(:data)
      
      def only_changes(old_parser)
        changes = {}
        @data.each do |name, new_won|
          changes[name] = new_won if !old_parser.data.has_key?(name) || old_parser.data[name] != new_won
        end
        changes
      end
      
      protected
      
      def process(stream)
        stream.each do |line|
          won, name = line.rstrip.split(/ +/)
          @data[name] = yield(won)
        end
      end
    end
    
    def self.process_stat_file(meta, old_stat_file_column, stream, data, player_data_column, &converter)
      parser = Parser.new(stream, &converter)
      old_parser = Parser.new(meta.method(old_stat_file_column).call, &converter)
      parser.only_changes(old_parser).each do |name, won|
        data[name].method("#{player_data_column}=").call(won)
      end
      meta.method("#{old_stat_file_column}=").call(stream)
    end
        
  end
end
