# NOMS -- A web-app for Armagetron Advanced statistics
# Copyright (C) 2010  Daniel Harple
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation in version 3 of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

module NOMS::Util
  def self.top_path(file)
    return File.expand_path(File.join(File.dirname(__FILE__), "..", "..", file))
  end
  
  def self.public_path(file)
    return File.join(top_path("public"), file)
  end
  
  def self.check_path(file)
    if file.index("..")
      raise RuntimeError.new(file)
    end
  end
  
  def self.e_sh(str)
  	str.to_s.gsub(/(?=[^a-zA-Z0-9_.\/\-\x7F-\xFF\n])/n, '\\').gsub(/\n/, "'\n'").sub(/^$/, "''")
  end
  
  def self.armasort_by(xs, &by)
    xs.sort_by { |x| by.call(x).gsub(/[^a-zA-Z1-9]/, "").downcase }
  end
  
  module RenameHelper
    def rename_helper(obj, old_name, new_name)
      data = obj[old_name]
      obj.delete(old_name)
      obj[new_name] = data
    end
  end
  
  class MethodNotMissing
    def initialize(*args)
    end

    def method_missing(name, *args, &blk)
      return self
    end
  end
  
  class ReverseFile < File
    def reverse_each(should_seek=true, max_bytes=4096)
      seek(0, IO::SEEK_END) if should_seek
      i = tell()
      j = nil
      buf = ""
      loop do
        k = buf.rindex("\n", -2)
        if k
          yield(buf.slice!(k + 1..-1))
        else
          if i.zero?
            yield(buf)
            return self
          end
          j, i = i, [0, i - max_bytes].max
          seek(i, IO::SEEK_SET)
          buf = read(j - i) + buf
        end
      end
    end
  end
end
