# NOMS -- A web-app for Armagetron Advanced statistics
# Copyright (C) 2009  Daniel Harple
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation in version 3 of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

$LOAD_PATH << File.dirname(__FILE__)
require "noms"
require "builder"

config = ENV["CONFIG"]

if config.nil?
  puts "No config specified. Use CONFIG=path/to/config.noms"
  exit(1)
end

NOMS::Setup.add_config(config)
NOMS::Setup.generate_models(false)

module NOMS
  def self.export_armaskill(path="armaskill.txt")
    File.open(path, "w") do |f|
      NOMS::DB::Player.each { |player| f.puts(player.rating) }
    end
    nil
  end
  
  def self.export_country_counts(path="country_counts.txt")
    map = NOMS::Choropleth::Map.new
    File.open(path, "w") do |f|
      map.countries.each do |id, country|
        f.puts("\"#{NOMS::CountryCode.country_name_from_alpha2(id)}\"\t#{country["count"]}")
      end
    end
    nil
  end
  
  def self.export_players
    NOMS::DB::Player.filter_data(NOMS::DB::Player).each do |player|
      puts player.name
    end
  end
  
  def self.export_team_skills(team, players)
    builder = Builder::XmlMarkup.new(:target => STDOUT, :indent => 4)
    players = players.map { |name| NOMS::DB::Player.find_by_name(name) }
    team_skill = players.inject(Struct.new(:mu, :sigma).new) { |skill, player|
      skill.mu ||= 0.0
      skill.sigma ||= 0.0
      skill.mu += player.mu
      skill.sigma += player.sigma
      skill
    }
    builder.team(:name => team, :mu => team_skill.mu, :sigma => team_skill.sigma) { |team|
      players.each do |player|
        team.player(player.display_name, :mu => player.mu, :sigma => player.sigma, :rating => player.rating)
      end
    }
  end
end
