// Copyright (c) 2009, Daniel Harple

var NOMS = new Object();
NOMS.GraphData = {};

NOMS.values = function(hash) {
    var xs = [];
    for (var k in hash)
        xs.push(hash[k]);
    return xs;
};

NOMS.show_tooltip = function(item) {
    var contents = item.series.label + ": " + item.datapoint[1] + " on " + item.datapoint[0];
    
    $('<div class="tooltip">' + contents + '</div>').css({
        top: item.pageY + 5,
        left: item.pageX + 5
    }).appendTo("body");
};

NOMS.armagetronad_escape_name = function(name) {
    // FIXME: Authenticated names are case sensitive, can't lowercase them.
    // name = name.toLowerCase();
    name = name.replace(" ", "_");
    return name;
};

NOMS.player_stats = function(names, succeed_func) {
    var ajaxSuccess = {};

    for (var i = 0; i < names.length; i++) {
        var name = names[i];
        if (this.GraphData[name]) {
            succeed_func();
            continue;
        }
        
        NOMS.request_data(name, succeed_func);
    }
};

NOMS.request_data = function(name, succeed_func) {
    NOMS.GraphData[name] = false;
    
    $.ajax({
        url: "match_history/player",
        dataType : "json",
        data: { name: name },
        success : function(data, textStatus) {
            NOMS.graph_player(name, data);
            if (jQuery.inArray(false, NOMS.values(NOMS.GraphData)) == -1)
                NOMS.draw_graph();
            succeed_func();
        },
        error : function(x, txt, e) {
            delete NOMS.GraphData[name];
            // TODO: better error reporting
            alert("Error: " + x.responseText);
        }
    });
    
};

NOMS.draw_graph = function() {
    var options = {
        legend: { show: true, container: $("#legend") },
        shadowSize: 0,
        lines: { show: true },
        grid: { hoverable: true }
    };
    var graphData = this.values(this.GraphData);
    this.normalize(graphData);
    var graph = $("#graph");
    var plot = $.plot(graph, graphData, options);
    
    var previousPoint = null;
    graph.bind("plothover", function (event, pos, item) {
        if (item) {
            if (previousPoint != item.datapoint) {
                previousPoint = item.datapoint;
                $(".tooltip").remove();
                NOMS.show_tooltip(item);
                // if ($("#show_all_tooltips").attr("checked"))
                //     AA.show_all_tooltips(plot, item);
            }
        }
        else {
            $(".tooltip").remove();
            previousPoint = null;            
        }
    });
    
};

NOMS.graph_player = function(name, stats) {
    stats.unshift([stats[0][0], 0]);
    this.GraphData[name] = {
        label: name,
        data: stats
    };
};

// Normalizes graph data, so that all data plots end on the same x-coordinate.
NOMS.normalize = function(data) {
    var xs = [];
    for (var i = 0; i < data.length; i++) {
        var plot = data[i]["data"];
        xs.push(plot[plot.length - 1][0]);
    }
    var maxX = Math.max.apply(Math, xs);
    
    for (var i = 0; i < data.length; i++) {
        var plot = data[i]["data"];
        var lastPoint = plot[plot.length - 1];
        
        // Extend the graph if needed
        if (maxX > lastPoint[0])
            plot.push([maxX, lastPoint[1]]);
    }
};

$(document).ready(function() {
    $("#name").focus();
    
    $("#entry-form").submit(function() {
        var name = $("#name");
        
        NOMS.player_stats(
            [NOMS.armagetronad_escape_name(name.val())],
            function() { name.val(""); }
        );
        return false;
    });
    
    $("#free_kill-vs-player_1").click(function() {
        NOMS.player_stats(["free_kill", "player_1"], function() {});
    });
    
    $("#top-match-wins").click(function() {
        // NOMS.player_stats(["b3n", "anumeric", "/dev/null", "xdude", "cracker"]);
        NOMS.player_stats(["b3n", "anumeric", "xdude", "/dev/null", "cracker", "troneater"], function() {});
    });
});
