// Copyright (c) 2009, Daniel Harple

var STAT_RANK_SMALLER_IS_BETTER = [];

function enabled_nodes(nodes) {
    var enabled = [];
    for (var i = 0; i < nodes.length; i++) {
        var node = $(nodes[i]);
        if (!node.hasClass("disabled"))
            enabled.push(node);
    }
    return enabled;
}

function insert_borders(header1, header2) {
    $(".column-border").removeClass("column-border");
    
    var general_headings = $("td", header1);
    var all_headings = $("th", header2);
    
    var enabled_general_headings = enabled_nodes(general_headings);
    var enabled_specific_headings = enabled_nodes(all_headings);
    
    var current_column = 0;
    var last_general_heading = null;
    var last_specific_heading = null;
    for (var i = 0; i < enabled_general_headings.length; i++) {
        var heading = $(enabled_general_headings[i]);
        var colspan = parseInt(heading.attr("colspan"));
        current_column += colspan;
        
        var specific_heading = enabled_specific_headings[current_column - 1];
        
        if (last_specific_heading) {
            $("." + heading_main_class(last_specific_heading)).addClass("column-border");
            last_general_heading.addClass("column-border");
        }
        
        last_specific_heading = specific_heading;
        last_general_heading = heading;        
    }
}

function insert_column_headers() {
    var header1 = $("#column_headers1");
    var header2 = $("#column_headers2");
    
    insert_borders(header1, header2);
    
    var stats = $("#stats > tbody > tr.stat");
    var stats_len = stats.length;
    
    var start_at = current_primary_table_sorting_direction() == 0 ? 20 : 19;
    for (var i = start_at; i < stats_len; i += 19) {
        var node = $(stats[i]);
        header2.clone().addClass("guide-headers").insertAfter(node);
        header1.clone().addClass("guide-headers no-borders").insertAfter(node);
    }
}

function remove_column_headers() {
    $("#stats > tbody > tr.guide-headers").remove();
}

function move_comparision_base_player() {
    var base_player_node = $("#base_player");
    if (base_player_node.length < 1)
        return;
    base_player_node.prependTo($("#stats tbody"));
}

function init_stat_table() {
    var stats_table = $("#stats");
    stats_table.tablesorter({
        sortList: default_sorting_order(),
        headers: {
            0: { sorter: false }
        }
    });
    
    stats_table.bind("sortStart",function() {
        remove_column_headers();
    });
    
    stats_table.bind("sortEnd",function() {
        insert_column_headers();
        number_ranks();
        move_comparision_base_player();
    });
    
    stats_table.trigger("update");
    stats_table.trigger("sorton", [default_sorting_order()]);
    // $("#stats").get(0).config.sortInitialOrder = "desc";
}

function current_primary_table_sorting_direction() {
    var stats_node = $("#stats");
    if (stats_node.length <= 0)
        return 0;
    var config = stats_node.get(0).config;
    if (config)
        return config.sortList[0][1];
    return 0;
}

function rank_smaller_is_better() {
    var index = $("#stats").get(0).config.sortList[0][0];
    var node = $("#column_headers2 th:eq(" + index  + ")");
    for (var i = 0; i < STAT_RANK_SMALLER_IS_BETTER.length; i++) {
        if (node.hasClass(STAT_RANK_SMALLER_IS_BETTER[i]))
            return true;
    }
    return false;
}

function toggle_stats_options_help() {
    var help = $("#stats_options_list");
    var desc = $("#stats_options_action_description");
    if (help.hasClass("hide")) {
        help.slideDown("fast");
        help.removeClass("hide");
        desc.text("hide");
    }
    else {
        help.slideUp("fast");
        help.addClass("hide");
        desc.text("expand");
    }
}

function generate_rank_function(nodes) {
    var rank;
    
    var biggest_first = 1;
    var smallest_first = 0;
    
    var sorting_direction = current_primary_table_sorting_direction();
    var smaller_is_better = rank_smaller_is_better();
    
    if ((!smaller_is_better && sorting_direction == biggest_first) ||
        (smaller_is_better  && sorting_direction == smallest_first))
    {
            
        rank = function(i) { return i + 1; };
    }
    else {
        var nodes_len = nodes.length;
        rank = function(i) { return nodes_len - i; };
    }    
    return rank;
}

function number_ranks() {
    var nodes = $("#stats > tbody > tr.stat[id!=base_player]");
    var rank = generate_rank_function(nodes);
    
    $.each(nodes, function(i, node) {
        $(".required-rank", node).text(rank(i));
    });
}

function default_sorting_order() {
    return [[parseInt($("#default_sorting_order_column").text()), 1]];
}

function heading_basename(name) {
    var parts = name.split("-");
    var basename = parts[0];
    return basename;
}

function heading_main_class(node) {
    var klasses = node.attr("class").split(" ");
    for (var i = 0; i < klasses.length; i++)
        if (klasses[i].match("-"))
            return klasses[i];
}

function adjust_general_heading_colspans(name, nodes, change_value) {
    var basename = heading_basename(name);
    var general_heading = $("#" + basename + "-" + "header");
    
    var old_colspan = parseInt(general_heading.attr("colspan"));
    var new_colspan = old_colspan + change_value;
    general_heading.attr("colspan", new_colspan);
    
    if (new_colspan == 0) {
        general_heading.hide();
        general_heading.addClass("disabled");
    }
    else {
        general_heading.show();
        general_heading.removeClass("disabled");
    }
    
    remove_column_headers();
    insert_column_headers();
}

function flash_node(node) {
    var old_color = node.css("background-color");
    node.animate( { backgroundColor: "#ffc" }, 500).animate( { backgroundColor: old_color }, 400);
}

function init_smaller_is_better() {
    $.each($("#smaller_is_better li"), function(i, node) {
        STAT_RANK_SMALLER_IS_BETTER.push($(node).text());
    });
}

function toggle_option(input_node) {
    var klass = "." + input_node.name;
    var nodes = $(klass);
    var specific_header = $($("th" + klass)[0]);
    if (input_node.checked) {
        nodes.show();
        nodes.removeClass("disabled");
        adjust_general_heading_colspans(input_node.name, nodes, 1);
        // flash_node(specific_header);
    }
    else {
        nodes.hide();
        nodes.addClass("disabled");
        adjust_general_heading_colspans(input_node.name, nodes, -1);
    }
}

$(document).ready(function () {
    
    init_smaller_is_better();
    
    $("#stats_options input").bind("click", function() {
        toggle_option(this);
    });
    
    $("#stats_options_action").bind("click", function() {
        toggle_stats_options_help();
    });
    
    $("#stats_options_hide").bind("click", function() {
        toggle_stats_options_help();
    });
    
    var stats_options = $("#stats_options .hide_options");
    stats_options.bind("click", function() {
        var toggle_when = function(option, set_to, should_toggle) {
            if (should_toggle) {
                option.checked = set_to;
                toggle_option(option);
            }
        };
        
        var any_enabled = false;
        var node = $(this).parent();
        var options = $("ul li input", node);
        for (var i = 0; i < options.length; i++) {
            if (options[i].checked) {
                any_enabled = true;
                break;
            }
        }
        for (var i = 0; i < options.length; i++) {
            var opt = options[i];
            if (any_enabled)
                toggle_when(opt, undefined, opt.checked);
            else
                toggle_when(opt, "checked", !opt.checked);
        }
    });
    
    if (parseInt($("#required-header").attr("colspan")) > 1) {
        init_stat_table();
    }
    else {
        remove_column_headers();
        insert_column_headers();
    }
});
